// Generated by CoffeeScript 1.10.0
(function() {
  var Appenders, ConsoleAppender, DefaultFormatter, FileAppender, Formatter, LogAppender, LogEntry, LogFormatHandler, LogFormats, LogHub, Logger, PassThrough, Transform, Writable, _, basic, dateFormat, fs, overlay, print, ref, util,
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  ref = require('stream'), PassThrough = ref.PassThrough, Transform = ref.Transform, Writable = ref.Writable;

  fs = require('fs-extra');

  _ = require('lodash');

  dateFormat = require('dateformat');

  basic = require('./anylog.config.predef').basic;

  util = require('util');

  print = function(object, depth, colorize) {
    if (depth == null) {
      depth = 2;
    }
    if (colorize == null) {
      colorize = false;
    }
    return util.inspect(object, false, depth, colorize);
  };

  overlay = function(a, b) {
    return _.merge(a, b, function(a, b) {
      if (_.isArray(a)) {
        return b;
      }
    });
  };

  LogEntry = (function() {
    function LogEntry(msg1, data1) {
      var ref1;
      this.msg = msg1;
      this.data = data1;
      this.tags = (ref1 = this.data) != null ? ref1.tags : void 0;
      delete this.data.tags;
    }

    return LogEntry;

  })();

  Formatter = (function() {
    function Formatter() {}

    Formatter.prototype.fromString = function(lvl, msg) {};

    Formatter.prototype.fromEntry = function(lvl, entry) {

      /*
        The formatter used by default if no other {Formatter} is specified for a destination.
       */
    };

    return Formatter;

  })();

  DefaultFormatter = (function(superClass) {
    var extractTags;

    extend(DefaultFormatter, superClass);

    function DefaultFormatter() {
      return DefaultFormatter.__super__.constructor.apply(this, arguments);
    }

    extractTags = function(tags) {
      if (_.isEmpty(tags)) {
        return "";
      } else {
        return " [" + tags + "]";
      }
    };

    DefaultFormatter.prototype.fromString = function(lvl, msg) {
      var entry, time;
      time = dateFormat(new Date(), "d/mm HH:MM:ss.l");
      return entry = "[" + time + "] " + (lvl.toUpperCase()) + " - " + msg + "\n";
    };

    DefaultFormatter.prototype.fromEntry = function(lvl, entry) {
      var data, tags, time;
      tags = extractTags(entry.tags);
      data = _.isEmpty(entry.data) ? "" : "(" + (print(entry.data, 3)) + ")";
      time = dateFormat(new Date(), "d/mm HH:MM:ss.l");
      return entry = "[" + time + "] " + (lvl.toUpperCase()) + tags + " - " + entry.msg + " " + data + "\n";
    };

    return DefaultFormatter;

  })(Formatter);


  /*
    Public registry for different log formats
   */

  LogFormats = (function() {
    function LogFormats() {}

    LogFormats.llog = function() {
      return new DefaultFormatter;
    };

    return LogFormats;

  })();


  /*
    Log appenders receive formatted log messages and route them to their sink them on their
    sink. The sink might by the processes stdout, a file, a database or any other type of destination.
    LogAppender is very similar to log4js' Appender or bunyan's Transport
   */

  LogAppender = (function() {
    function LogAppender(opts) {
      this.sink = opts != null ? opts.sink : void 0;
    }

    LogAppender.prototype.initialize = function() {};

    return LogAppender;

  })();

  FileAppender = (function(superClass) {
    extend(FileAppender, superClass);

    function FileAppender(opts) {
      FileAppender.__super__.constructor.call(this, {
        sink: fs.createWriteStream(opts.filename, {
          flags: 'a'
        })
      });
    }

    return FileAppender;

  })(LogAppender);

  ConsoleAppender = (function(superClass) {
    extend(ConsoleAppender, superClass);

    function ConsoleAppender() {
      ConsoleAppender.__super__.constructor.call(this, {
        sink: process.stdout
      });
    }

    return ConsoleAppender;

  })(LogAppender);


  /*
    Apply log formatters to log messages before they are passed to configured {LogAppender}s
    @private
   */

  LogFormatHandler = (function(superClass) {
    extend(LogFormatHandler, superClass);

    function LogFormatHandler(formatter1, level1) {
      this.formatter = formatter1;
      this.level = level1;
      LogFormatHandler.__super__.constructor.call(this, {
        objectMode: true
      });
      this.setMaxListeners(100);
    }

    LogFormatHandler.prototype._transform = function(chunk, enc, next) {
      var msg;
      msg = (function() {
        switch (false) {
          case chunk.constructor !== String:
            return this.formatter.fromString(this.level, chunk);
          case !(chunk instanceof Buffer):
            return this.formatter.fromString(this.level, chunk.toString());
          case !(chunk instanceof LogEntry):
            return this.formatter.fromEntry(this.level, chunk);
          default:
            return "Unexpected type of log message " + chunk;
        }
      }).call(this);
      this.push(msg);
      return next();
    };

    return LogFormatHandler;

  })(Transform);

  Appenders = (function() {
    var instance;

    instance = null;

    Appenders.get = function() {
      return instance != null ? instance : instance = new Appenders;
    };

    function Appenders(registry) {
      this.registry = registry != null ? registry : {};
      this.register('console', ConsoleAppender);
      this.register('file', FileAppender);
    }

    Appenders.prototype.register = function(alias, factory) {
      return this.registry[alias] = factory;
    };

    Appenders.prototype.create = function(def) {
      return new this.registry[def.type](def);
    };

    return Appenders;

  })();


  /*
    Central class for log message processing. The hub receives log messages for specific log levels,
    applies formatting (via {LogFormatHandler}) and streams into the connected {LogAppender}s.
  
    All log message processing is based on streams. Formatters are transform streams, appenders are writable
    streams.
   */

  LogHub = (function() {
    LogHub.defaultOpts = function() {
      return {
        appenders: Appenders.get(),
        destinations: [
          {
            appender: {
              type: 'console'
            },
            levels: ['info', 'warn', 'error', 'debug']
          }
        ],
        levels: ['info', 'warn', 'error', 'debug']
      };
    };

    function LogHub(opts) {
      if (opts == null) {
        opts = {};
      }
      this.opts = overlay(LogHub.defaultOpts(), opts);
      this.initialize();
    }

    LogHub.prototype.initialize = function() {
      var connector, destination, i, j, len, len1, level, ref1, ref2, results;
      if (this.opts.basedir) {
        fs.mkdirsSync(this.opts.basedir);
      }
      this.dispatcher = {};
      ref1 = this.opts.levels;
      for (i = 0, len = ref1.length; i < len; i++) {
        level = ref1[i];
        connector = new PassThrough({
          objectMode: true
        });
        connector.setMaxListeners(100);
        this.dispatcher[level] = connector;
      }
      ref2 = this.opts.destinations;
      results = [];
      for (j = 0, len1 = ref2.length; j < len1; j++) {
        destination = ref2[j];
        results.push(this.addDestination(destination));
      }
      return results;
    };

    LogHub.prototype.addDestination = function(destination) {
      var appender, formatter, i, len, level, ref1, results;
      appender = (function() {
        switch (false) {
          case destination.appender.type.constructor !== String:
            return this.opts.appenders.create(destination.appender);
          case !(destination.appender.type instanceof Function):
            return new destination.appender.type(destination.appender);
          default:
            throw new Error("Unknown specification of appender type: " + destination.appender.type);
        }
      }).call(this);
      ref1 = destination.levels;
      results = [];
      for (i = 0, len = ref1.length; i < len; i++) {
        level = ref1[i];
        if (!this.dispatcher[level]) {
          console.log("WARNING: Log level " + level + " not defined but requested by destination " + (print(destination)));
        }
        formatter = destination.formatter || LogFormats.llog();
        appender.sink.setMaxListeners(50);
        results.push(this.dispatcher[level].pipe(new LogFormatHandler(formatter, level)).pipe(appender.sink));
      }
      return results;
    };

    LogHub.prototype.log = function(lvl, msg, data) {
      var ref1, ref2;
      if (data) {
        return (ref1 = this.dispatcher[lvl]) != null ? ref1.push(new LogEntry(msg, data)) : void 0;
      } else {
        return (ref2 = this.dispatcher[lvl]) != null ? ref2.push(msg) : void 0;
      }
    };

    LogHub.prototype.logger = function() {
      return new Logger(this.opts.levels, this);
    };

    return LogHub;

  })();


  /*
    Wrapper around {LogHub} that provides a method for each available log level.
    This allows for convenient use of the existential operator to guard log statements from
    ever being executed.
  
    Note: Using the existential operator does not only prevent the log message from being sent
    to the appenders but actually prevents the message from being constructed! This allows to make
    heavy use of debug logging without introducing any GC overhead into production code.
  
    @example Lazy logging to log level 'debug'
      logger = hub.logger()
      logger.debug? "This String is only created if log level debug exists", {meta: "The same for any metadata"}
   */

  Logger = (function() {
    var logHandler;

    logHandler = function(lvl, hub) {
      return function(msg, data) {
        return hub.log(lvl, msg, data);
      };
    };

    function Logger(levels, hub1) {
      var i, len, level;
      if (levels == null) {
        levels = [];
      }
      this.hub = hub1;
      for (i = 0, len = levels.length; i < len; i++) {
        level = levels[i];
        this[level] = logHandler(level, this.hub);
      }
    }

    return Logger;

  })();

  module.exports = {
    LogHub: LogHub,
    LogAppender: LogAppender,
    FileAppender: FileAppender,
    ConsoleAppender: ConsoleAppender,
    ConfigPredef: require('./anylog.config.predef')
  };

}).call(this);
