stream = require 'stream'
{LogHub, LogAppender} = require '../src/anylog'
_ = require 'lodash'

class CountingStream extends stream.Transform

  constructor: (@cnt = 0) -> super

  _transform: (chunk, enc, next) ->
    @cnt++
    @push chunk
    next()

class LogStream extends stream.Writable

  constructor: (@shouldLog = true) -> super

  _write: (chunk, enc, next) ->
    console.log chunk.toString() if @shouldLog
    next()

class RecordingStream extends stream.Writable

  constructor: ->
    @records = []
    super


  _write: (chunk, enc, next) ->
    @records.push chunk.toString()
    next()

  find: (chunk) ->
    result = _.filter @records, (item) -> (item.indexOf chunk) > -1
    result

describe 'anylog', ->

  it '# handles different log levels independently', ->
    info = new CountingStream
    error = new CountingStream
    warn = new CountingStream
    debug = new CountingStream
    hub = new LogHub destinations : [
      {appender:
        type : LogAppender
        sink : info
      levels : ['info']}
      {appender:
        type : LogAppender
        sink : error
      levels : ['error']}
      {appender:
        type : LogAppender
        sink : warn
      levels : ['warn']}
      {appender:
        type : LogAppender
        sink : debug
      levels : ['debug']}

    ]
    hub.log "info", "Some message"
    hub.log "warn", "Some message"
    hub.log "debug", "Some message"
    hub.log "error", "Some message"
    expect(info.cnt).to.equal(1)
    expect(error.cnt).to.equal(1)
    expect(debug.cnt).to.equal(1)
    expect(warn.cnt).to.equal(1)

  it '# different log levels can be aggregated', ->
    all = new CountingStream
    hub = new LogHub destinations : [
      {appender:
        type : LogAppender
        sink : all
      levels : ['info', 'warn', 'debug', 'error']}
    ]
    hub.log "info", "Some message"
    hub.log "warn", "Some message"
    hub.log "debug", "Some message"
    hub.log "error", "Some message"
    expect(all.cnt).to.equal(4)


  it '# exposes a logger with methods for each defined log level', ->
    all = new CountingStream
    hub = new LogHub destinations : [
      {
        appender:
          type : LogAppender
          sink: all
        levels : ['info', 'warn', 'debug', 'error']
      }
    ]
    log = hub.logger()
    log.info "Some message"
    log.warn "Some message"
    log.debug "Some message"
    log.error "Some message"
    log.trace? "This log level does not exists"
    expect(all.cnt).to.equal(4)

  it '# supports logging of tags as a special form of additional data', ->
    records = new RecordingStream
    logStream = new LogStream
    hub = new LogHub destinations : [
      {
        appender:
          type : LogAppender
          sink: records
        levels : ['info', 'warn', 'debug', 'error']
      }
      {
        appender:
          type : LogAppender
          sink: logStream
        levels : ['info', 'warn', 'debug', 'error']
      }
    ]
    hub.logger().info 'My message', tags : ['Streaming','AnotherTag'], production:true for i in [1..10]
    expect(records.find '[Streaming,AnotherTag]').to.have.length 10
